from types import ModuleType
from typing import Callable
from typing_extensions import Any
from .stubs import _LoginMethodTransfer

def init_log(m: ModuleType, /) -> None: ...

__version__: Any
__build__: Any

class Client(Any): ...

class Account:
    event_callbacks: list[Callable[[Any], Any]]
    def __init__(
        self, uin: int, data_folder: str, protocol: str  # TODO: Literal
    ) -> None: ...
    async def login(self, method: _LoginMethodTransfer) -> Client: ...



from graia.amnesia.message import Element
from enum import Enum
from dataclasses import dataclass
from typing import Literal
from typing_extensions import Self
from functools import total_ordering
from graia.amnesia.message.element import Text as Text


@dataclass
class At(Element):
    target: int
    display: str | None = None

    def __str__(self) -> str:
        return f"@{self.target}"


class AtAll(Element):
    def __str__(self) -> str:
        return "@全体成员"

    def __repr__(self) -> str:
        return "AtAll()"


class FingerGuessing(Element):
    @total_ordering
    class Choice(Enum):
        Rock = "石头"
        Scissors = "剪刀"
        Paper = "布"

        def __eq__(self, other: Self) -> bool:
            if not isinstance(other, FingerGuessing.Choice):
                raise TypeError(f"{other} 不是 FingerGuessing.Choice")
            return self.value == other.value

        def __lt__(self, other: Self) -> bool:
            if not isinstance(other, FingerGuessing.Choice):
                raise TypeError(f"{other} 不是 FingerGuessing.Choice")
            return (self.name, other.name) in {
                ("Rock", "Scissors"),
                ("Scissors", "Paper"),
                ("Paper", "Rock"),
            }

    choice: Choice

    def __init__(
        self,
        choice: Literal["Rock", "Paper", "Scissors" "石头", "剪刀", "布"] | Choice,
    ) -> None:
        C = FingerGuessing.Choice
        if isinstance(choice, str) and choice in ("Rock", "Paper", "Scissors"):
            self.choice = C[choice]
        if isinstance(choice, C):
            self.choice = choice
        raise TypeError(f"无效的猜拳参数：{choice}")

    def __str__(self) -> str:
        return f"[猜拳: {self.choice.value}]"

    def __repr__(self) -> str:
        return f"FingerGuessing(choice={self.choice})"


class Dice(Element):
    value: Literal[1, 2, 3, 4, 5, 6]

    def __init__(self, value: Literal[1, 2, 3, 4, 5, 6]) -> None:
        if value not in range(1, 6 + 1):
            raise ValueError(f"{value} 不是有效的骰子值")
        self.value = value

    def __str__(self) -> str:
        return f"[骰子: {self.value}]"

    def __repr__(self) -> str:
        return f"Dice(value={self.value})"



from __future__ import annotations
from . import ichika as lib
from .stubs import _LoginMethodTransfer

lib.init_log(lib)

__version__ = lib.__version__
__build__ = lib.__build__
Account = lib.Account


class LoginMethod:
    QRCode = _LoginMethodTransfer("""{"type": "QRCode"}""")

    @staticmethod
    def Password(
        password: str, md5: bool = False, sms: bool = False
    ) -> _LoginMethodTransfer:
        import json

        return _LoginMethodTransfer(
            json.dumps(
                {"type": "Password", "password": password, "md5": md5, "sms": sms}
            )
        )
